#!/bin/sh
###
# EncFS-ROX integration - Change password from encrypted directory
# Author: James Budiono - Copyright James Budiono 2010
# License: GNU GPL v3 or newer
# v0.4 - fix symlink check
###

if [ -z "$1" ]; then
	Xdialog --msgbox "Run this tool from within ROX-Filer." 0 0
	exit
fi

# can't process symlink
if [ "$(file "$1" | grep "symbolic link")" ]; then
	Xdialog --title "Error" --msgbox "Symbolic link is not supported." 0 0
	exit
fi

# check - encfsctl exist? no ==> quit
if [ -z "$(which encfsctl)" ]; then
	Xdialog --title "Error" --msgbox "EncFS not found - cannot continue. Please install EncFS first." 0 0
	exit
fi

# 1. prepare all the vars
ENCFS_SIG=".encfs6.xml"    
ORIG_DIR="$1"
PREFIX="${ORIG_DIR%/*}/"
DIR="${ORIG_DIR##*/}"

ENC_DIR="${PREFIX}.${DIR}"

# checks - are we trying to open an encrypted storage directory?
if [ -f "${ORIG_DIR}/${ENCFS_SIG}" ]; then
	Xdialog --title "Error" --msgbox "This is a storage directory for encrypted files, it cannot be opened." 0 0
	exit
fi

# checks - does storage for encrypted dir exist, with a valid signature? 
if [ ! -f "${ENC_DIR}/${ENCFS_SIG}" ]; then
	Xdialog --title "Error" --msgbox "This directory is not encrypted." 0 0
	exit
fi

# 2. Get the 3 sets of password - original, new, and new-reconfirm
PASS_OK=no
PASS0=$(Xdialog --stdout --title "Old Password" --password --inputbox "Type your existing password" 0 0)
if [ -n "$PASS0" ]; then

	PASS1="x"
	while [ "$PASS1" != "$PASS2" ]; do
		PASS1=$(Xdialog --stdout --title "New Pwd" --password --inputbox "Type in a new password" 0 0)
		if [ ! $? -eq 0 ]; then 
			Xdialog --title "Cancel" --msgbox "Nothing is changed." 0 0 
			exit
		fi
		if [ -z "$PASS1" ]; then 
			Xdialog --msgbox "Password cannot be empty. Please re-enter." 0 0
			PASS1="x"
			continue;
		fi
		PASS2=$(Xdialog --stdout --title "Confirm" --password --inputbox "Re-confirm new password" 0 0)
		if [ ! $? -eq 0 ]; then 
			Xdialog --title "Cancel" --msgbox "Nothing is changed." 0 0 
			exit
		fi
		
	done
	if Xdialog --title "WARNING" --ok-label OK --cancel-label Cancel --yesno \
			"Please ensure you can remember the password.
			Please keep it in a safe place and don't lose it.
			If you forget it, you cannot access your files EVER again.
			If this is fine, click OK to continue, otherwise click Cancel" 0 0; then 
			PASS_OK="yes"
	fi
fi


# 3. Change the password
if [ "$PASS_OK" = "yes" ]; then
	cat <<EOF | encfsctl autopasswd "${ENC_DIR}"
$PASS0
$PASS1
EOF

	if [ $? -eq 0 ]; then
		Xdialog --title "Success!" --msgbox "Password changed successfully.
						Please remember to keep your password in a safe place." 0 0 
	else
		Xdialog --title "Error" --msgbox "Unable to change password.
		Did you type the existing password correctly?" 0 0 
	fi

else
	Xdialog --title "Cancel" --msgbox "Nothing is changed." 0 0 
fi
